import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/analytics/helpers/format_number_to_kwh.dart';
import 'package:syncrow_web/pages/analytics/helpers/get_month_name_from_int.dart';
import 'package:syncrow_web/pages/analytics/models/energy_data_model.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

// energy_consumption_chart will return id, name and consumption
const phasesJson = {
  "1": {
    "phaseOne": 1000,
    "phaseTwo": 2000,
    "phaseThree": 3000,
  }
};

class TotalEnergyConsumptionChart extends StatelessWidget {
  const TotalEnergyConsumptionChart({required this.chartData, super.key});

  final List<EnergyDataModel> chartData;

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: LineChart(
        LineChartData(
          titlesData: _titlesData(context),
          lineBarsData: _lineBarsData,
          gridData: FlGridData(
            show: true,
            drawVerticalLine: false,
            drawHorizontalLine: true,
          ),
          borderData: FlBorderData(
            show: true,
            border: const Border.symmetric(
              horizontal: BorderSide(
                color: ColorsManager.greyColor,
                width: 1,
                style: BorderStyle.solid,
              ),
            ),
          ),
          lineTouchData: LineTouchData(
            handleBuiltInTouches: true,
            touchSpotThreshold: 2,
            touchTooltipData: _lineTouchTooltipData(),
          ),
        ),
        duration: Durations.extralong1,
        curve: Curves.easeIn,
      ),
    );
  }

  List<LineChartBarData> get _lineBarsData {
    return [
      LineChartBarData(
        preventCurveOvershootingThreshold: 0.1,
        curveSmoothness: 0.55,
        preventCurveOverShooting: true,
        spots: chartData
            .asMap()
            .entries
            .map(
              (entry) => FlSpot(
                entry.key.toDouble(),
                entry.value.value,
              ),
            )
            .toList(),
        color: ColorsManager.blueColor.withValues(alpha: 0.6),
        shadow: Shadow(color: Colors.black12),
        show: true,
        isCurved: true,
        belowBarData: BarAreaData(
          show: true,
          gradient: LinearGradient(
            colors: [
              ColorsManager.vividBlue.withValues(alpha: 0.3),
              ColorsManager.vividBlue.withValues(alpha: 0.2),
              ColorsManager.vividBlue.withValues(alpha: 0.1),
              Colors.transparent,
            ],
            begin: Alignment.center,
            end: Alignment.bottomCenter,
          ),
        ),
        dotData: FlDotData(show: false),
        isStrokeCapRound: true,
        barWidth: 3,
      ),
    ];
  }

  FlTitlesData _titlesData(BuildContext context) {
    return FlTitlesData(
      show: true,
      bottomTitles: AxisTitles(
        drawBelowEverything: true,
        sideTitles: SideTitles(
          interval: 1,
          reservedSize: 32,
          showTitles: true,
          maxIncluded: true,
          getTitlesWidget: (value, meta) => Padding(
            padding: const EdgeInsets.only(top: 20.0),
            child: Text(
              chartData.elementAtOrNull(value.toInt())?.date.month.getMonthName ??
                  '',
              style: context.textTheme.bodySmall?.copyWith(
                fontSize: 12,
                color: ColorsManager.greyColor,
              ),
            ),
          ),
        ),
      ),
      leftTitles: AxisTitles(
        sideTitles: SideTitles(
          showTitles: true,
          maxIncluded: true,
          reservedSize: 110,
          getTitlesWidget: (value, meta) => Padding(
            padding: const EdgeInsetsDirectional.only(end: 12),
            child: FittedBox(
              fit: BoxFit.scaleDown,
              child: Text(
                value.formatNumberToKwh,
                style: context.textTheme.bodySmall?.copyWith(
                  fontSize: 12,
                  color: ColorsManager.greyColor,
                ),
              ),
            ),
          ),
        ),
      ),
      rightTitles: AxisTitles(sideTitles: SideTitles(showTitles: false)),
      topTitles: AxisTitles(sideTitles: SideTitles(showTitles: false)),
    );
  }

  LineTouchTooltipData _lineTouchTooltipData() {
    return LineTouchTooltipData(
      getTooltipColor: (touchTooltipItem) => ColorsManager.whiteColors,
      tooltipBorder: const BorderSide(color: ColorsManager.semiTransparentBlack),
      tooltipRoundedRadius: 16,
      showOnTopOfTheChartBoxArea: false,
      tooltipPadding: const EdgeInsets.all(8.0),
      getTooltipItems: _getTooltipItems,
    );
  }

  List<LineTooltipItem?> _getTooltipItems(List<LineBarSpot> touchedSpots) {
    return touchedSpots.map((spot) {
      return LineTooltipItem(
        _getToolTipLabel(spot.x + 1, spot.y),
        const TextStyle(
          color: ColorsManager.textPrimaryColor,
          fontWeight: FontWeight.w600,
          fontSize: 12,
        ),
      );
    }).toList();
  }

  String _getToolTipLabel(num month, double value) {
    final monthLabel = month.getMonthName;
    final valueLabel = value.formatNumberToKwh;
    final labels = [monthLabel, valueLabel];
    return labels.where((element) => element.isNotEmpty).join(', ');
  }
}
